//
//  URLLoader.m
//  TwitterViewer
//
//  Created by 大田 貴之 on 11/06/07.
//  Copyright 2011 Apple. All rights reserved.
//

#import <Foundation/Foundation.h>

#import "URLLoader.h"

@implementation URLLoader

@synthesize connection;
@synthesize data;

// （1）
/*
 最初のレスポンス受信時に1回だけ呼ばれ、受信処理の初期化などを行います。サンプルでは受信データの格納先であるNSMutableDataインスタンスを初期化しています。
 */
- (void) connection:(NSURLConnection *)connection 
 didReceiveResponse:(NSURLResponse *)response {
    self.data = [NSMutableData data];
}

// （2）
/*
 データ受信時に呼ばれます。ここで受け取るデータは受信途中の断片的なデータです。通信が終わるまでに何度も呼ばれます。そのため、サンプルでは通信が終わるまで フィールドのNSMutableDataに受信データを累積させています。
 */
-(void) connection:(NSURLConnection *)connection 
    didReceiveData:(NSData *)receiveData {
    [self.data appendData:receiveData];
}

// （3）
/*
 通信完了時に1回だけ呼ばれます。
 */
- (void) connectionDidFinishLoading:(NSURLConnection *)connection {
    [[NSNotificationCenter defaultCenter] 
     postNotificationName: @"connectionDidFinishNotification" 
     object: self];
}

// （4）
/*
 通信エラー発生時に呼ばれます。
 */
- (void) connection:(NSURLConnection *)connection 
   didFailWithError:(NSError *)error {
    [[NSNotificationCenter defaultCenter] 
     postNotificationName: @"connectionDidFailWithError" 
     object: self];
}

// （5）
/*
 　URLLoaderクラスが、ほかのクラスから利用される際に呼ばれるメソッドです。NSURLConnectionインスタンスのデリゲートとしてURLLoaderのインスタンス自身を設定し、初期化しています。
 NSURLConnectionでは、connectionWithRequest:delegateメソッドで初期化を行った場合、特に開始メソッドなどを呼ばなくても、非同期通信が開始します。
 */
- (void) loadFromUrl: (NSString *)url method: (NSString *) method {
    NSMutableURLRequest *req = 
    [NSMutableURLRequest requestWithURL:[NSURL URLWithString:url]];
    [req setHTTPMethod:method];
    self.connection = [NSURLConnection connectionWithRequest:req delegate:self];
}

- (void)dealloc {
    [connection release];
    [data release];
    [super dealloc];
}

@end